@@ -2,7 +2,7 @@
    This file is part of kalarmcal library, which provides access to KAlarm
    calendar data.
 
-   SPDX-FileCopyrightText: 2018-2024 David Jarvie <djarvie@kde.org>
+   SPDX-FileCopyrightText: 2018-2025 David Jarvie <djarvie@kde.org>
 
    SPDX-License-Identifier: LGPL-2.0-or-later
 */
@@ -16,9 +16,12 @@ using namespace KAlarmCal;
 #include <KCalendarCore/Event>
 #include <KCalendarCore/Alarm>
 using namespace KCalendarCore;
+#include <KHolidays/HolidayRegion>
 #include <KLocalizedString>
 
 #include <QTest>
+#include <QFileInfo>
+#include <QTextStream>
 
 QTEST_GUILESS_MAIN(KAEventTest)
 
@@ -444,6 +447,7 @@ void KAEventTest::flags()
         QVERIFY(!event.commandDisplay());
         QVERIFY(!event.reminderOnceOnly());
         QVERIFY(!event.emailBcc());
+        KAEvent::setHolidays();
     }
     {
         const KAEvent::Flags flags(KAEvent::COPY_KORGANIZER | KAEvent::WORK_TIME_ONLY);
@@ -1301,28 +1305,229 @@ void KAEventTest::setNextOccurrence()
     const KADateTime dt(QDate(2005, 10, 29), QTime(1, 30, 0), QTimeZone("Europe/London"));
     KAEvent event(dt, QStringLiteral("name"), QStringLiteral("text"), Qt::black, Qt::white, QFont(), KAEvent::SubAction::Message, 0, KAEvent::DEFAULT_FONT);
     event.setRecurDaily(1, QBitArray(7, true), -1, QDate());
-    KAEvent::OccurType type = event.setNextOccurrence(dt);
+    KAEvent::OccurType type;
+    bool updated = event.setNextOccurrence(dt, type);
     const DateTime next1 = event.mainDateTime();
-    QCOMPARE(type, KAEvent::OccurType::RecurDateTime);
+    QVERIFY(updated);
+    QCOMPARE(type, KAEvent::OccurType::Recur);
     QCOMPARE(next1.date(), QDate(2005, 10, 30));
     QCOMPARE(next1.effectiveTime(), QTime(1, 30, 0));
     KAEvent eventUTC = event;
 
     const KADateTime dt1(QDate(2005, 10, 30), QTime(1, 30, 0), QTimeZone("Europe/London"));
-    type = event.setNextOccurrence(next1.kDateTime());
+    updated = event.setNextOccurrence(next1.kDateTime(), type);
     const DateTime next2 = event.mainDateTime();
-    QCOMPARE(type, KAEvent::OccurType::RecurDateTime);
+    QVERIFY(updated);
+    QCOMPARE(type, KAEvent::OccurType::Recur);
     QCOMPARE(next2.date(), QDate(2005, 10, 31));
     QCOMPARE(next2.effectiveTime(), QTime(1, 30, 0));
 
     const KADateTime dt2(QDate(2005, 10, 30), QTime(1, 30, 0), KADateTime::UTC);
-    type = eventUTC.setNextOccurrence(dt2);
+    updated = eventUTC.setNextOccurrence(dt2, type);
     const DateTime next3 = eventUTC.mainDateTime();
-    QCOMPARE(type, KAEvent::OccurType::RecurDateTime);
+    QVERIFY(updated);
+    QCOMPARE(type, KAEvent::OccurType::Recur);
     QCOMPARE(next3.date(), QDate(2005, 10, 31));
     QVERIFY(!next3.isSecondOccurrence());
 }
 
+void KAEventTest::nextDateTime()
+{
+    // Test behaviour of working time only, excluding holidays and excluding date/time
+    // works as expected, for recurrences and for sub-repetitions.
+
+    // NOTE: Because the Holidays class only deals with dates in the future,
+    //       we have to find a future year whose dates are on the correct 
+    //       days of the week for this test.
+    int year;
+    for (year = QDate::currentDate().year() + 1;  QDate(year, 6, 3).dayOfWeek() != 1;  ++year) ;
+    const KADateTime dtMon_6_3(QDate(year, 6, 3), QTime(12, 0, 0), QTimeZone("Europe/London"));
+    const KADateTime dtThu_6_13(QDate(year, 6, 13), QTime(12, 0, 0), QTimeZone("Europe/London"));
+    const KADateTime dtSunHol_6_23(QDate(year, 6, 23), QTime(12, 0, 0), QTimeZone("Europe/London"));
+    const KADateTime dtWedHol_7_3(QDate(year, 7, 3), QTime(12, 0, 0), QTimeZone("Europe/London"));
+    const KADateTime dtSat_7_13(QDate(year, 7, 13), QTime(12, 0, 0), QTimeZone("Europe/London"));
+    const KADateTime dtTue_7_23(QDate(year, 7, 23), QTime(12, 0, 0), QTimeZone("Europe/London"));
+    const KADateTime dtFri_8_2(QDate(year, 8, 2), QTime(12, 0, 0), QTimeZone("Europe/London"));
+    const KADateTime dtThu_8_22(QDate(year, 8, 22), QTime(12, 0, 0), QTimeZone("Europe/London"));
+
+    // Set KAEvent holidays to June 15 and June 23.
+    QTemporaryDir dir;
+    QVERIFY(dir.isValid());
+    const QString path = dir.path();
+    QFile holidayFile(path + QStringLiteral("/holiday_gb-eaw_en-gb_Test"));
+    QVERIFY(holidayFile.open(QIODeviceBase::WriteOnly));
+    QTextStream fStream(&holidayFile);
+    fStream << "country     \"GB-EAW\"\n"
+               "language    \"en_GB\"\n"
+               "name        \"England and Wales\"\n"
+               "description \"Test holiday file\"\n\n"
+               "\"Holiday Wed Rep\" public on june 5\n"
+               "\"Holiday Sun Rec\" public on june 23\n"
+               "\"Holiday Wed Rec\" public on july 3\n"
+               "\"Holiday Sun Rep\" public on august 4\n";
+    holidayFile.close();
+    KHolidays::HolidayRegion hRegion{QFileInfo(holidayFile)};
+    QVERIFY(hRegion.isValid());
+    const auto holidayList = hRegion.rawHolidays(QDate(year,1,1), QDate(year,12,31));
+    QCOMPARE(holidayList.count(), 4);
+    static Holidays holidays(hRegion);
+    KAEvent::setHolidays(holidays);
+    QVERIFY(holidays.isHoliday(QDate(year, 6, 5)));
+    QVERIFY(holidays.isHoliday(QDate(year, 6, 23)));
+    QVERIFY(holidays.isHoliday(QDate(year, 7, 3)));
+    QVERIFY(holidays.isHoliday(QDate(year, 8, 4)));
+
+    // Set KAEvent working days to Monday - Friday, 9am - 5pm.
+    QBitArray workDays(7, false);
+    workDays.fill(true, 0, 5);
+    KAEvent::setWorkTime(workDays, QTime(9,0,0), QTime(17,0,0), KADateTime::LocalZone);
+
+    // Set event recurrence to start June 3, recur every 10 days,
+    // with one sub-repetition 2 days later.
+    KAEvent event(dtMon_6_3, QStringLiteral("name"), QStringLiteral("text"), Qt::black, Qt::white, QFont(), KAEvent::SubAction::Message, 0, KAEvent::DEFAULT_FONT);
+    event.setRecurDaily(10, QBitArray(7, true), -1, QDate());  // recur every 10 days
+    event.setRepetition(Repetition(Duration(2, Duration::Days), 1));   // 1 sub-rep after 2 days
+    DateTime next;
+
+    // Get next recurrence/sub-repetition.
+    KAEvent::TriggerType type = event.nextDateTime(dtSunHol_6_23.addSecs(-60), next, KAEvent::NextRepeat);
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtSunHol_6_23);
+    type = event.nextDateTime(dtSunHol_6_23, next, KAEvent::NextRepeat);
+    QVERIFY(KAEvent::isRepeat(type));
+    QCOMPARE(next.kDateTime(), dtSunHol_6_23.addDays(2));
+    type = event.nextDateTime(dtSunHol_6_23.addDays(2), next, KAEvent::NextRepeat);
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtSunHol_6_23.addDays(10));
+
+    // Exclude a recurrence date: returns next recurrence, not sub-repetition
+    event.setExceptionDates({dtSunHol_6_23.date()});
+    type = event.nextDateTime(dtSunHol_6_23.addSecs(-60), next, KAEvent::NextRepeat);
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtSunHol_6_23.addDays(10));
+    type = event.nextDateTime(dtSunHol_6_23, next, KAEvent::NextRepeat);
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtSunHol_6_23.addDays(10));
+    type = event.nextDateTime(dtSunHol_6_23.addDays(2), next, KAEvent::NextRepeat);
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtSunHol_6_23.addDays(10));
+
+    event.setExceptionDates({});   // clear the exception
+    type = event.nextDateTime(dtSunHol_6_23.addSecs(-60), next, KAEvent::NextRepeat);
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtSunHol_6_23);
+
+    // Working time only: returns recurrence, or sub-repetition only if the recurrence
+    // actually triggered.
+    event.setWorkTimeOnly(true);
+    // Check recurrence unaffected by working time still triggers
+    type = event.nextDateTime(dtMon_6_3.addSecs(-60), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QVERIFY(KAEvent::isFirstRecur(type));
+    QCOMPARE(next.kDateTime(), dtMon_6_3);
+    type = event.nextDateTime(dtMon_6_3, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QVERIFY(KAEvent::isRepeat(type));
+    QCOMPARE(next.kDateTime(), dtMon_6_3.addDays(2));
+    // Recurrence, and therefore its sub-repetition, suppressed by working time
+    type = event.nextDateTime(dtSunHol_6_23.addSecs(-60), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtSunHol_6_23.addDays(10));
+    type = event.nextDateTime(dtSunHol_6_23, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtSunHol_6_23.addDays(10));
+    // Individual sub-repetition suppressed by working time
+    event.setRepetition(Repetition(Duration(2, Duration::Days), 2));   // 2 sub-reps after 2 days
+    type = event.nextDateTime(dtThu_6_13, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QVERIFY(KAEvent::isRepeat(type));
+    QCOMPARE(next.kDateTime(), dtThu_6_13.addDays(4));
+    event.setRepetition(Repetition(Duration(2, Duration::Days), 1));   // 1 sub-rep after 2 days
+    event.setWorkTimeOnly(false);
+
+    // Holidays excluded: returns recurrence, or sub-repetition only if the recurrence
+    // actually triggered.
+    event.setExcludeHolidays(true);
+    // Check recurrence unaffected by holidays still triggers
+    type = event.nextDateTime(dtThu_8_22.addSecs(-60), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtThu_8_22);
+    type = event.nextDateTime(dtThu_8_22, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QVERIFY(KAEvent::isRepeat(type));
+    QCOMPARE(next.kDateTime(), dtThu_8_22.addDays(2));
+    type = event.nextDateTime(dtThu_8_22.addDays(2), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtThu_8_22.addDays(10));
+    // Recurrence, and therefore its sub-repetition, suppressed by holiday
+    QVERIFY(holidays.isHoliday(dtWedHol_7_3.date()));
+    type = event.nextDateTime(dtWedHol_7_3.addSecs(-60), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtWedHol_7_3.addDays(10));
+    type = event.nextDateTime(dtWedHol_7_3, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtWedHol_7_3.addDays(10));
+    // Individual sub-repetition suppressed by holiday
+    type = event.nextDateTime(dtMon_6_3.addSecs(-60), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QVERIFY(KAEvent::isFirstRecur(type));
+    QCOMPARE(next.kDateTime(), dtMon_6_3);
+    type = event.nextDateTime(dtMon_6_3, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtMon_6_3.addDays(10));
+    event.setExcludeHolidays(false);
+
+    // Work time only, and holidays excluded: returns recurrence, or sub-repetition only
+    // if the recurrence actually triggered.
+    event.setWorkTimeOnly(true);
+    event.setExcludeHolidays(true);
+    // Check recurrence unaffected by working time or holidays still triggers
+    type = event.nextDateTime(dtTue_7_23.addSecs(-60), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtTue_7_23);
+    type = event.nextDateTime(dtTue_7_23, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QVERIFY(KAEvent::isRepeat(type));
+    QCOMPARE(next.kDateTime(), dtTue_7_23.addDays(2));
+    type = event.nextDateTime(dtTue_7_23.addDays(2), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtTue_7_23.addDays(10));
+    // Recurrence, and therefore its sub-repetition, suppressed by holiday
+    type = event.nextDateTime(dtWedHol_7_3.addSecs(-60), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtWedHol_7_3.addDays(20));
+    type = event.nextDateTime(dtWedHol_7_3, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtWedHol_7_3.addDays(20));
+    // Recurrence, and therefore its sub-repetition, suppressed by working time
+    type = event.nextDateTime(dtSat_7_13.addSecs(-60), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtSat_7_13.addDays(10));
+    type = event.nextDateTime(dtSat_7_13, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtSat_7_13.addDays(10));
+    // Recurrence, and therefore its sub-repetition, suppressed by both working time
+    // and holiday
+    type = event.nextDateTime(dtSunHol_6_23.addSecs(-60), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtTue_7_23);
+    type = event.nextDateTime(dtSunHol_6_23, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtTue_7_23);
+    // Individual sub-repetition suppressed by working time
+    type = event.nextDateTime(dtThu_8_22, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtThu_8_22.addDays(20));
+    // Individual sub-repetition suppressed by holiday
+    type = event.nextDateTime(dtMon_6_3.addSecs(-60), next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QVERIFY(KAEvent::isFirstRecur(type));
+    QCOMPARE(next.kDateTime(), dtMon_6_3);
+    // Individual sub-repetition suppressed by both working time and holiday
+    type = event.nextDateTime(dtFri_8_2, next, (KAEvent::NextRepeat | KAEvent::NextWorkHoliday));
+    QCOMPARE(type, KAEvent::TriggerType::Recur);
+    QCOMPARE(next.kDateTime(), dtFri_8_2.addDays(10));
+
+//TODO: Test for date-only alarms
+
+    event.setWorkTimeOnly(false);
+    event.setExcludeHolidays(false);
+    KAEvent::setHolidays();
+}
+
 #include "moc_kaeventtest.cpp"
 
 // vim: et sw=4:
